
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity dramCtrl is
	 Generic (
				-- defines wheter we simulate or implement the design. This is needed by the MCB rtl
				-- allowed values: "TRUE" and "FALSE"
				IS_SIMULATION		: string := "FALSE";
	 
				-- defines how long one burst (= block transfer) on the wishbone master interface is
				-- data blocks contain WB_BURST_LEN words at maximum. Between two blocks is some time 
				-- (approx. 9 cycles of WbClkxCI) in which data is loaded from the dram. Lower values 
				-- give a very bad bus utilization.
				WB_BURST_LEN		: integer := 16	-- range 2 .. 64 
	 );
    Port ( 
				-- system interface
				ResetSwitchxR	: in  STD_LOGIC;	-- reset switch input (this comes from a push button)
	 			RstSocxRI		: in  STD_LOGIC;	-- logic reset for wishbone bus
				RstCorexRI		: in  STD_LOGIC;	-- logic reset for analyzer core (sampling clock domain)
				--RstxRO			: out STD_LOGIC;	-- system reset output (held high during pll locking)
				CalibDonexSO	: out STD_LOGIC;	-- get high when the dram has been initialized and is ready to receive data
				-- clocks
				BoardClkxCI 	: in  STD_LOGIC;	-- positive of differential on board clock (200MHz)
				--BoardClkxCIB	: in  STD_LOGIC;   	-- negative of differential on board clock (200MHz)
				RefClkxCO		: out STD_LOGIC;	-- PLL generated reference clock for the system (25MHz)
				-- interface to sampling core
				CoreClkxC		: in  std_logic;	   	-- sampling system clock (100MHz or 200MHz)
				DataxDI 		: in  std_logic_vector(31 downto 0);          
				WritexSI		: in  std_logic;
				-- wishbone dma master interface
				WbClkxCI		: in  std_logic;	-- wishbone clock (master & slave)
				MaCycxSO		: out std_logic;
				MaStbxSO		: out std_logic;
				MaWexSO			: out std_logic;
				MaSelxDO		: out std_logic_vector(3 downto 0);
				MaAdrxDO		: out std_logic_vector(31 downto 0);
				MaDatxDO		: out std_logic_vector(31 downto 0);
				MaDatxDI		: in  std_logic_vector(31 downto 0);
				MaAckxSI		: in  std_logic;
				MaErrxSI		: in  std_logic;
				-- wishbone configuration slave interface
				SlCycxSI		: in  std_logic;
				SlStbxSI		: in  std_logic;
				SlWexSI			: in  std_logic;
				SlAdrxDI		: in  std_logic_vector(4 downto 2);
				SlDatxDI		: in  std_logic_vector(31 downto 0);
				SlDatxDO		: out std_logic_vector(31 downto 0);
				SlAckxSO		: out std_logic;
				IntxSO			: out std_logic;
				-- physical memory interface
				MemDQxDIO		: inout std_logic_vector(15 downto 0);
				MemAdrxDO		: out	  std_logic_vector(12 downto 0);
				MemBaxDO		: out   std_logic_vector(2 downto 0);
				MemRasxEO		: out	  std_logic;
				MemCasxEO		: out   std_logic;
				MemWexEO		: out   std_logic;
				MemOdtxO		: out	  std_logic;
				MemCkexEO		: out   std_logic;
				MemClkxCO		: out   std_logic;
				MemClkxCOB		: out	  std_logic;
				MemLDqsxEO		: inout std_logic;
				MemLDqsxEOB		: inout std_logic;
				MemUDqsxEO		: inout std_logic;
				MemUDqsxEOB		: inout std_logic;
				MemLdmxEO		: out	  std_logic;
				MemUdmxEO		: out   std_logic;
				-- calibration (not actually logical signals)
				RzqxIO			: inout std_logic;
				ZioxIO			: inout std_logic				
			 );
end dramCtrl;

architecture Behavioral of dramCtrl is

-- Memory controller generated by Ip Core Memory Interface Generator --
component DdrMcb
 generic(
    C3_P0_MASK_SIZE           : integer := 4;
    C3_P0_DATA_PORT_SIZE      : integer := 32;
    C3_P1_MASK_SIZE           : integer := 4;
    C3_P1_DATA_PORT_SIZE      : integer := 32;
    C3_MEMCLK_PERIOD          : integer := 3200;
    C3_RST_ACT_LOW            : integer := 1;
    C3_INPUT_CLK_TYPE         : string := "DIFFERENTIAL";
    C3_CALIB_SOFT_IP          : string := "TRUE";
    C3_SIMULATION             : string := "TRUE";
    DEBUG_EN                  : integer := 0;
    C3_MEM_ADDR_ORDER         : string := "ROW_BANK_COLUMN";
    C3_NUM_DQ_PINS            : integer := 16;
    C3_MEM_ADDR_WIDTH         : integer := 13;
    C3_MEM_BANKADDR_WIDTH     : integer := 3
);
    port (
   mcb3_dram_dq                            : inout  std_logic_vector(C3_NUM_DQ_PINS-1 downto 0);
   mcb3_dram_a                             : out std_logic_vector(C3_MEM_ADDR_WIDTH-1 downto 0);
   mcb3_dram_ba                            : out std_logic_vector(C3_MEM_BANKADDR_WIDTH-1 downto 0);
   mcb3_dram_ras_n                         : out std_logic;
   mcb3_dram_cas_n                         : out std_logic;
   mcb3_dram_we_n                          : out std_logic;
   mcb3_dram_odt                           : out std_logic;
   mcb3_dram_cke                           : out std_logic;
   mcb3_dram_dm                            : out std_logic;
   mcb3_dram_udqs                          : inout  std_logic;
   mcb3_dram_udqs_n                        : inout  std_logic;
   mcb3_rzq                                : inout  std_logic;
   mcb3_zio                                : inout  std_logic;
   mcb3_dram_udm                           : out std_logic;
   c3_sys_clk							   : in std_logic;
   --c3_sys_clk_p                            : in  std_logic;
   --c3_sys_clk_n                            : in  std_logic;
   c3_sys_rst_n                            : in  std_logic;
   c3_calib_done                           : out std_logic;
   c3_clk0                                 : out std_logic;
   c3_rst0                                 : out std_logic;
   mcb3_dram_dqs                           : inout  std_logic;
   mcb3_dram_dqs_n                         : inout  std_logic;
   mcb3_dram_ck                            : out std_logic;
   mcb3_dram_ck_n                          : out std_logic;
   c3_p0_cmd_clk                           : in std_logic;
   c3_p0_cmd_en                            : in std_logic;
   c3_p0_cmd_instr                         : in std_logic_vector(2 downto 0);
   c3_p0_cmd_bl                            : in std_logic_vector(5 downto 0);
   c3_p0_cmd_byte_addr                     : in std_logic_vector(29 downto 0);
   c3_p0_cmd_empty                         : out std_logic;
   c3_p0_cmd_full                          : out std_logic;
   c3_p0_wr_clk                            : in std_logic;
   c3_p0_wr_en                             : in std_logic;
   c3_p0_wr_mask                           : in std_logic_vector(C3_P0_MASK_SIZE - 1 downto 0);
   c3_p0_wr_data                           : in std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0);
   c3_p0_wr_full                           : out std_logic;
   c3_p0_wr_empty                          : out std_logic;
   c3_p0_wr_count                          : out std_logic_vector(6 downto 0);
   c3_p0_wr_underrun                       : out std_logic;
   c3_p0_wr_error                          : out std_logic;
   c3_p0_rd_clk                            : in std_logic;
   c3_p0_rd_en                             : in std_logic;
   c3_p0_rd_data                           : out std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0);
   c3_p0_rd_full                           : out std_logic;
   c3_p0_rd_empty                          : out std_logic;
   c3_p0_rd_count                          : out std_logic_vector(6 downto 0);
   c3_p0_rd_overflow                       : out std_logic;
   c3_p0_rd_error                          : out std_logic;
	c3_p1_cmd_clk                           : in std_logic;
   c3_p1_cmd_en                            : in std_logic;
   c3_p1_cmd_instr                         : in std_logic_vector(2 downto 0);
   c3_p1_cmd_bl                            : in std_logic_vector(5 downto 0);
   c3_p1_cmd_byte_addr                     : in std_logic_vector(29 downto 0);
   c3_p1_cmd_empty                         : out std_logic;
   c3_p1_cmd_full                          : out std_logic;
   c3_p1_wr_clk                            : in std_logic;
   c3_p1_wr_en                             : in std_logic;
   c3_p1_wr_mask                           : in std_logic_vector(C3_P1_MASK_SIZE - 1 downto 0);
   c3_p1_wr_data                           : in std_logic_vector(C3_P1_DATA_PORT_SIZE - 1 downto 0);
   c3_p1_wr_full                           : out std_logic;
   c3_p1_wr_empty                          : out std_logic;
   c3_p1_wr_count                          : out std_logic_vector(6 downto 0);
   c3_p1_wr_underrun                       : out std_logic;
   c3_p1_wr_error                          : out std_logic;
   c3_p1_rd_clk                            : in std_logic;
   c3_p1_rd_en                             : in std_logic;
   c3_p1_rd_data                           : out std_logic_vector(C3_P1_DATA_PORT_SIZE - 1 downto 0);
   c3_p1_rd_full                           : out std_logic;
   c3_p1_rd_empty                          : out std_logic;
   c3_p1_rd_count                          : out std_logic_vector(6 downto 0);
   c3_p1_rd_overflow                       : out std_logic;
   c3_p1_rd_error                          : out std_logic
	);
	end component;
	
	constant DRAM_CNTR_WIDTH			: integer := 27;
	constant ZERO_VECTOR				: std_logic_vector(31 downto 0) := (others => '0');
	
	Signal ResetSwitchxRB				: std_logic;
	Signal CalibDonexSN, CalibDonexSP	: std_logic := '0';
	Signal CalibDonexS					: std_logic;
	
	signal RefClkxC						: std_logic;
	type WriteFsmType is (fsmIdle, fsmWIssue);
	Signal WrFsmStatexSN, WrFsmStatexSP		: WriteFsmType;
	Signal WCmdValidxS					: std_logic;
	Signal WCmdInstrxS					: std_logic_vector(2 downto 0);
	Signal WBurstLenxD					: std_logic_vector(5 downto 0);
	Signal WCmdEmptyxS					: std_logic;
	Signal WCmdFullxS					: std_logic;
	Signal WEmptyxSN, WEmptyxSP			: std_logic; -- core clock domain
	Signal WEmptySynxSN, WEmptySynxSP   : std_logic; -- avr clock domain
	Signal WrFifoCntxDN, WrFifoCntxDP	: integer range 0 to 63 := 0;
	--Signal WrFifoCntxDN, WrFifoCntxDP	: integer range -8 to 64 := 0;	-- although values below 0 never occur -8 is neccessary for simulation in modelsim
	Signal ByteAdrxDN, ByteAdrxDP		: std_logic_vector(DRAM_CNTR_WIDTH-1 downto 0);
	Signal DramByteAdrxD				: std_logic_vector(29 downto 0);
	Signal SamplePtrxDN, SamplePtrxDP	: std_logic_vector(DRAM_CNTR_WIDTH-1 downto 0); -- original signal in core clock domain
	Signal SamplePtrSynxDN, SamplePtrSynxDP : std_logic_vector(DRAM_CNTR_WIDTH-1 downto 0);	-- avr clock domain
	Signal RDatOutxD					: std_logic_vector(31 downto 0);
	Signal RConsumexS					: std_logic;
	Signal RCmdValidxS					: std_logic;
	Signal REmptyxS						: std_logic;
	Signal RBurstLenxD					: std_logic_vector(5 downto 0);
	Signal RCmdFullxS					: std_logic;
	Signal RCmdEmptyxS					: std_logic;
	-- error signals from mcb
	Signal RErrorxS, ROverflowxS		: std_logic;
	Signal WErrorxS, WUnderrunxS		: std_logic;
	
	-- configuration registers for dma transfer
	Signal DestAdrxDN, DestAdrxDP		: std_logic_vector(15 downto 0);
	Signal SrcAdrxDN, SrcAdrxDP			: std_logic_vector(DRAM_CNTR_WIDTH-1 downto 0);
	--Signal IncDestAdrxS				: std_logic;
	Signal DmaLenxDN, DmaLenxDP			: std_logic_vector(15 downto 0);
	Signal BusyFlagxS					: std_logic;
	Signal IEFlagxSN, IEFlagxSP			: std_logic;
	Signal ErrFlagxSN, ErrFlagxSP		: std_logic;
	Signal SetErrFlagxS					: std_logic;
	Signal DoneFlagxSN, DoneFlagxSP  	: std_logic;
	Signal SetDoneFlagxS				: std_logic;
	--Signal ClrWrAdrxSN, ClrWrAdrxSP	: std_logic;
	
	-- wishbone signals
	type   wbWriteFsmType is (wbFsmIdle, wbFsmFlushWrFifo, wbFsmWaitFlush, wbFsmIssue, wbFsmWaitDram, wbAccessBus, wbWaitAck);
	Signal WbStatexSN, WbStatexSP		: wbWriteFsmType;
	Signal MaDatxDN, MaDatxDP			: std_logic_vector(31 downto 0);
	Signal WbBurstCntxDN, WbBurstCntxDP	: integer range -1 to WB_BURST_LEN+1 := 0;
	Signal MaCycxSN, MaCycxSP			: std_logic;
	Signal MaStbxSN, MaStbxSP			: std_logic;
	Signal SlDatOutxDN, SlDatOutxDP		: std_logic_vector(31 downto 0);
	Signal SlAckxSN, SlAckxSP			: std_logic;
	
	-- control signals crossing clock domain
	Signal FlushWrFifoxSN, FlushWrFifoxSP	: std_logic;
	
begin

	ResetSwitchxRB	<= not ResetSwitchxR;
--	CalibDonexSO	<= CalibDonexSP;
	CalibDonexSO	<= CalibDonexS;
	RefClkxCO 		<= RefClkxC;
	
	-- insert a flip flop into the reset line from Mcb to system for timing reasons
--	process (RefClkxC)
--	begin
--		if RefClkxC'event and RefClkxC='1' then
----			RstxRP			<= RstxRN;
--			CalibDonexSP 	<= CalibDonexSN;
--		end if;
--	end process;
	
	-- wishbone interfaces logic
	IntxSO			<= IEFlagxSP and (DoneFlagxSP or ErrFlagxSP);
	MaAdrxDO 		<= x"0000" & DestAdrxDP;
	MaSelxDO		<= "1111";
	MaDatxDO 		<= RDatOutxD;	-- foreward mcb output data to master data output
	RConsumexS 		<= MaAckxSI;	-- once the slave acked we don't need the data anymore
	BusyFlagxS		<= '1' when conv_integer(unsigned(DmaLenxDP))/=0 else '0';
	-- set error flag if a bus error is detected on the master interface
	SetErrFlagxS 	<= MaCycxSP and MaStbxSP and MaErrxSI;
	-- set done flag when last dma transfer is done
	SetDoneFlagxS 	<= '1' when DmaLenxDP=x"0001" and RConsumexS='1' else '0';
	SlDatxDO		<= SlDatOutxDP;
	SlAckxSO		<= SlAckxSP;
	MaDatxDN		<= RDatOutxD;
	
	-- check for write access on configuration interface
	lenCnt : process (DmaLenxDP, DestAdrxDP, SrcAdrxDP, SlCycxSI, SlStbxSI, SlWexSI, SlAdrxDI, SlDatxDI, SetErrFlagxS, SetDoneFlagxS, ErrFlagxSP, IEFlagxSP, DoneFlagxSP, RConsumexS)
	begin
		DmaLenxDN 	<= DmaLenxDP;
		DestAdrxDN	<= DestAdrxDP;
		SrcAdrxDN 	<= SrcAdrxDP;
		ErrFlagxSN 	<= ErrFlagxSP or SetErrFlagxS;
		IEFlagxSN 	<= IEFlagxSP;
		DoneFlagxSN <= DoneFlagxSP or SetDoneFlagxS;
		--ClrWrAdrxSN	<= ClrWrAdrxSP;
		
		if SlCycxSI='1' and SlStbxSI='1' and SlWexSI='1' then
			case (SlAdrxDi(4 downto 2)) is
				when "000" => -- config and status reg
					IEFlagxSN <= SlDatxDI(1);
					if SlDatxDI(2)='1' then
						DoneFlagxSN <= '0';
					end if;
					if SlDatxDI(3)='1' then
						ErrFlagxSN <= '0';
					end if;
				when "001" => -- src address reg
					SrcAdrxDN <= SlDatxDI(DRAM_CNTR_WIDTH-1 downto 2) & "00"; -- make sure the address is word aligned
				when "010" => -- dest address reg
					DestAdrxDN <= SlDatxDI(15 downto 2) & "00";	-- make sure the address is word aligned
				when "011" => -- length register
					DmaLenxDN <= SlDatxDI(15 downto 0);
				when others =>
			end case;
		end if;
		if RConsumexS='1' then	-- read consume means one word was transfered over the dma channel
			DmaLenxDN	<= std_logic_vector(conv_unsigned(conv_integer(unsigned(DmaLenxDP))-1,16));
			-- increment the source and destination addresses one word (=4 bytes)
			DestAdrxDN <= std_logic_vector(conv_unsigned(conv_integer(unsigned(DestAdrxDP))+4,16));
			SrcAdrxDN  <= std_logic_vector(conv_unsigned(conv_integer(unsigned(SrcAdrxDP))+4,DRAM_CNTR_WIDTH));
		end if;	
	end process;
	
	SamplePtrSynxDN <= SamplePtrxDP;
	process(SlAdrxDI, ErrFlagxSP, DoneFlagxSP, IEFlagxSP, BusyFlagxS, SrcAdrxDP, DestAdrxDP, DmaLenxDP, SamplePtrSynxDP, WrFifoCntxDP)
	begin
		-- data output multiplexer 
		case (SlAdrxDi(4 downto 2)) is
			when "000" => -- config and status reg
				--SlDatOutxDN <=  std_logic_vector(conv_unsigned(WrFifoCntxDP,8)) & x"000" & "000" & RErrorxS &ROverflowxS & REmptyxS & FlushWrFifoxSN & WEmptyxSP & ErrFlagxSP & DoneFlagxSP & IEFlagxSP & BusyFlagxS;
				SlDatOutxDN <= x"0000000" & ErrFlagxSP & DoneFlagxSP & IEFlagxSP & BusyFlagxS;
			when "001" =>
				SlDatOutxDN <= ZERO_VECTOR(31-DRAM_CNTR_WIDTH downto 0) & SrcAdrxDP;
			when "010" =>
				SlDatOutxDN <= x"0000" & DestAdrxDP;
			when "011" =>
				SlDatOutxDN <= x"0000" & DmaLenxDP;
			when "100" =>
				SlDatOutxDN <= ZERO_VECTOR(31-DRAM_CNTR_WIDTH downto 0) & SamplePtrSynxDP;
			when others =>	
				SlDatOutxDN <= (others => '0');
		end case;
	end process;
		
	-- ack signal generation
	process (SlCycxSI, SlStbxSI, SlAckxSP)
	begin
		SlAckxSN <= '0';
		if SlCycxSI='1' and SlStbxSI='1' and SlAckxSP='0' then
			SlAckxSN <= '1';
		end if;
	end process;
	
	-- wishbone master interface
	MaCycxSO <= MaCycxSP;
	MaStbxSO <= MaStbxSP;
	MaWexSO  <= '1';
	WEmptySynxSN <= WEmptyxSP;	-- synchronize Write fifo empty signal to avr clock domain
	dmaMasterInterf : process(DmaLenxDP, WbBurstCntxDP, WbStatexSP, REmptyxS, WEmptySynxSP, RCmdFullxS, MaAckxSI, MaCycxSP)
	begin
		MaCycxSN		<= '0';
		MaStbxSN		<= '0';
		--IncDestAdrxS	<= '0';
		WbBurstCntxDN 	<= WbBurstCntxDP;
		WbStatexSN		<= WbStatexSP;
		FlushWrFifoxSN 	<= '0';
		RCmdValidxS		<= '0';
		RBurstLenxD		<= (others => '0');
		
		case (WbStatexSP) is
			when wbFsmIdle =>
				WbBurstCntxDN	<= 0;
				if DmaLenxDP /= x"0000" then	-- dma transfer starts if len is not zero
					if WEmptySynxSP='0' then	-- if the write fifo is not empty it has to be flushed first
						WbStatexSN <= wbFsmFlushWrFifo;
					else
						if RCmdFullxS='0' then
							WbStatexSN <= wbFsmIssue;
						end if;
					end if;
				end if;
				
			when wbFsmFlushWrFifo =>	-- wait until write fsm has flushed the write fifo
				FlushWrFifoxSN <= '1';
				if WEmptySynxSP='1' and RCmdFullxS='0' then
					WbStatexSN <= wbFsmIssue;
				end if;
			
			when wbFsmIssue =>	-- send read command to memory controller block
				RCmdValidxS	<= '1';
				WbStatexSN	<= wbFsmWaitDram;
				if WB_BURST_LEN < conv_integer(unsigned(DmaLenxDP)) then
					-- we still have more bytes to copy than the max burst len -> do max burst len
					RBurstLenxD 	<=	std_logic_vector(conv_unsigned(WB_BURST_LEN-1,6));
					WbBurstCntxDN 	<= WB_BURST_LEN;
				else
					-- only transfer the number of words left for this dma transfer
					RBurstLenxD		<= std_logic_vector(conv_unsigned(conv_integer(unsigned(DmaLenxDP))-1,6));
					WbBurstCntxDN 	<= conv_integer(unsigned(DmaLenxDP));
				end if;

			when wbFsmWaitDram =>
				MaCycxSN		<= MaCycxSP;		-- claim the bus if this is not the first transfer in this burst
				if REmptyxS='0' then
					WbStatexSN <= wbAccessBus;
				end if;
				
			when wbAccessBus =>
				MaCycxSN		<= '1';
				MaStbxSN 	<= '1';
				WbStatexSN 	<= wbWaitAck;
				WbBurstCntxDN <= WbBurstCntxDP-1;
				
			when wbWaitAck =>
				MaCycxSN		<= '1';
				MaStbxSN 	<= '1';
				if MaAckxSI='1' then
----------------------------				
-- Note by Lukas: This looked fine in a simulation but caused problems in the actual implementation. (number of bytes transferd to sram was wrong)
-- which resulted in wrong data being sent to the host. 
-- The reason for this remained unknown.
--
--					-- speed up things, if the mcb has data available just send it
--					if REmptyxS='0' and WbBurstCntxDP/=0 then
--						-- note: new output data is transfered to the output automatically by the incoming ack
--						--WbStatexSN <= wbAccessBus;
--						WbBurstCntxDN <= WbBurstCntxDP-1;
--					else
-- End of curious section
----------------------------
						MaStbxSN	<= '0';
						if WbBurstCntxDP=0 then
							-- we are done with this burst -> deassert cyc
							MaCycxSN <= '0';
							WbStatexSN <= wbFsmIdle;
						else
							WbStatexSN <= wbFsmWaitDram;
						end if;
--					end if;	
				end if;
				
			when others =>
				WbStatexSN	<= wbFsmIdle;
		end case;
	end process;

		
	-- wishbone clock domain registers
	wbClkRegs : process (WbClkxCI, RstSocxRI)
	begin
		if RstSocxRI='1' then
			DestAdrxDP		<= (others => '0');
			SrcAdrxDP		<= (others => '0');
			DmaLenxDP		<= (others => '0');
			IEFlagxSP		<= '0';
			ErrFlagxSP		<= '0';
			DoneFlagxSP		<= '0';
			WbStatexSP		<= wbFsmIdle;
			MaDatxDP		<= (others => '0');
			MaCycxSP		<= '0';
			MaStbxSP		<= '0';
			SlDatOutxDP		<= (others => '0');
			SlAckxSP		<= '0';
			FlushWrFifoxSP 	<= '0';
			WbBurstCntxDP 	<= 0;
			WEmptySynxSP	<= '0';
			SamplePtrSynxDP <= (others => '0');
		elsif WbClkxCI'event and WbClkxCI='1' then
			DestAdrxDP 		<= DestAdrxDN;
			SrcAdrxDP		<= SrcAdrxDN;
			DmaLenxDP		<= DmaLenxDN;
			IEFlagxSP		<= IEFlagxSN;
			ErrFlagxSP		<= ErrFlagxSN;
			DoneFlagxSP		<= DoneFlagxSN;
			WbStatexSP		<= WbStatexSN;
			MaDatxDP		<= MaDatxDN;
			MaCycxSP		<= MaCycxSN;
			MaStbxSP		<= MaStbxSN;
			SlDatOutxDP		<= SlDatOutxDN;
			SlAckxSP		<= SlAckxSN;
			FlushWrFifoxSP 	<= FlushWrFifoxSN;
			WbBurstCntxDP	<= WbBurstCntxDN;
			WEmptySynxSP	<= WEmptySynxSN;
			SamplePtrSynxDP <= SamplePtrSynxDN;
		end if;
	end process;

	-- state machine
	fsmComb : process (WrFsmStatexSP, WrFifoCntxDP, WCmdFullxS, FlushWrFifoxSP)
	begin
		-- defaults
		WrFsmStatexSN	<= WrFsmStatexSP;
		WCmdInstrxS		<= "000";
		WCmdValidxS		<= '0';
		WBurstLenxD		<= (others => '0');
				
		case (WrFsmStatexSP) is
			when fsmIdle =>
				if FlushWrFifoxSP='1' and WCmdFullxS='0' then	-- check for read request from analyzer core
					if WrFifoCntxDP/=0 then
						-- prior to reading we have to write all data from the fifo to the memory
						WrFsmStatexSN	<= fsmWIssue;
					end if;
				else
					-- if some bytes are in the fifo start a write
					if WrFifoCntxDP > 7 and WCmdFullxS='0' then
						WrFsmStatexSN	<= fsmWIssue;
					end if;
				end if;
							
			when fsmWIssue =>	-- queue a write command to the memory
				-- Note: a burstLen of 0 means the mem controller sends 1 word
				WBurstLenxD		<= std_logic_vector(conv_unsigned(WrFifoCntxDP-1,6));
				WCmdInstrxS		<= "000";
				WCmdValidxS		<= '1';
				WrFsmStatexSN	<= fsmIdle;
				
			when others =>	-- failsave operation
				WrFsmStatexSN	<= fsmIdle;
		end case;
	end process;

	-- calculate the dram address at which the next sample will be stored
	dramSamplePtr : process(ByteAdrxDP, WrFifoCntxDP)
		-- we use variables to make the code more readable
		variable adr		: integer range 0 to 2**DRAM_CNTR_WIDTH;
	begin
		adr 			:= conv_integer(unsigned(ByteAdrxDP));
		-- each sample stored in the fifo is 4 bytes long so we have to multiply with 4
		SamplePtrxDN	<= std_logic_vector(conv_unsigned(adr+(WrFifoCntxDP*4),DRAM_CNTR_WIDTH));
	end process;

	-- count the number of words stored in the write fifo which
	-- where not marked for transmission in the command fifo
	fifoCnt : process (WritexSI, WrFifoCntxDP, WBurstLenxD, WCmdInstrxS, WCmdValidxS)
	begin
		WrFifoCntxDN <= WrFifoCntxDP; -- default assignment
		-- count the arriving bytes, subtract the bytes issued in write commands to the mcb
		if WritexSI='1' then
			if WCmdInstrxS(0)='0' and WCmdValidxS='1' then	-- the lowest command bit indicates a write
				-- Note: a burst length of 0 means we write 1 word, 1 means 2 words and so on
				WrFifoCntxDN <= WrFifoCntxDP - (conv_integer(unsigned(WBurstLenxD))+1) + 1;
			else			
				WrFifoCntxDN <= WrFifoCntxDP + 1;
			end if;
		else
			if WCmdInstrxS(0)='0' and WCmdValidxS='1' then
				WrFifoCntxDN <= WrFifoCntxDP - (conv_integer(unsigned(WBurstLenxD))+1);
			end if;
		end if;	
	end process;
	
	-- dram address generation
	dramAddrCnt : process (ByteAdrxDP, WBurstLenxD, WCmdInstrxS, WCmdValidxS)
		-- we use variables to make the code more readable
		variable adr			: integer range 0 to ((128*1024*1024)-1);
		variable newAdr		: integer range 0 to ((128*1024*1024)+1024); -- some space for calculations
		variable burstLen		: integer range 0 to 64;
	begin
		ByteAdrxDN	<= ByteAdrxDP;	-- default: counter doesn't change
		adr			:= conv_integer(unsigned(ByteAdrxDP));
		-- a burst len of 0 means we transfer one word, BusrtLenxD=1 means 2 words and so on
		burstLen		:= conv_integer(unsigned(WBurstLenxD))+1;
		
		-- check for write commands sent to the memory
		if WCmdInstrxS(0)='0' and WCmdValidxS='1' then		-- the lowest command bit indicates read or write
			-- we have a 32 bit port -> one write takes 4 bytes so we multiply burstLen with 4
			newAdr		:= adr + (burstLen*4);
			-- check for overflow
			if newAdr >= 128*1024*1024 then
				newAdr	:= newAdr - (128*1024*1024);
			end if;
			ByteAdrxDN	<= std_logic_vector(conv_unsigned(newAdr,DRAM_CNTR_WIDTH));
		end if;
	end process;

	-- registers
	dramCtrlRxPortRegs : process (CoreClkxC, RstCorexRI)
	begin
		if RstCorexRI='1' then
			WrFifoCntxDP 	<= 0;
			WrFsmStatexSP	<= fsmIdle;
			SamplePtrxDP	<= (others => '0');
			WEmptyxSP       <= '0';
		elsif CoreClkxC'event and CoreClkxC='1' then
			WrFifoCntxDP 	<= WrFifoCntxDN;
			WrFsmStatexSP	<= WrFsmStatexSN;
			SamplePtrxDP	<= SamplePtrxDN;
			WEmptyxSP       <= WEmptyxSN;
		end if;
	end process;

	genStdAdrCnt : if IS_SIMULATION="FALSE" generate
		process (CoreClkxC, RstCorexRI)
		begin
			if RstCorexRI='1' then
				ByteAdrxDP		<= (others => '0');
			elsif CoreClkxC'event and CoreClkxC='1' then
				ByteAdrxDP		<= ByteAdrxDN;	
			end if;
		end process;
	end generate;
	genSimAdrCnt : if IS_SIMULATION="TRUE" generate
		process (CoreClkxC, RstCorexRI)
		begin
			if RstCorexRI='1' then
				ByteAdrxDP		<= "111"&x"fffff0";	-- used to test wrap around in the simulation
			elsif CoreClkxC'event and CoreClkxC='1' then
				ByteAdrxDP		<= ByteAdrxDN;	
			end if;		
		end process;	
	end generate;

	DramByteAdrxD <= ZERO_VECTOR(29-DRAM_CNTR_WIDTH downto 0) & ByteAdrxDP;

	-- the hardware ddr dram controller (coregen component)
	Inst_mcb : DdrMcb
    generic map (
		C3_SIMULATION     => IS_SIMULATION,
		C3_INPUT_CLK_TYPE => "SINGLE_ENDED"
	 )	
	port map (
    -- general signals
    --c3_sys_clk_p  		=>	BoardClkxCI,
	--c3_sys_clk_n    	=>  BoardClkxCIB,
	c3_sys_clk			=>  BoardClkxCI,
	c3_sys_rst_n    	=>  ResetSwitchxRB,                        
	-- memory interface
    mcb3_dram_dq	    =>  MemDQxDIO,  
    mcb3_dram_a        	=>  MemAdrxDO,  
    mcb3_dram_ba       	=>  MemBaxDO,
    mcb3_dram_ras_n    	=>  MemRasxEO,                        
    mcb3_dram_cas_n    	=>  MemCasxEO,                        
    mcb3_dram_we_n     	=>  MemWexEO,                          
    mcb3_dram_odt      	=>  MemOdtxO,                         
    mcb3_dram_cke      	=>  MemCkexEO,                          
    mcb3_dram_ck       	=>  MemClkxCO,                          
    mcb3_dram_ck_n     	=>  MemClkxCOB,       
    mcb3_dram_dqs      	=>  MemLDqsxEO,                          
    mcb3_dram_dqs_n  	=>  MemLDqsxEOB,
    mcb3_dram_udqs  	=>  MemUDqsxEO,             
    mcb3_dram_udqs_n	=>  MemUDqsxEOB, 
    mcb3_dram_udm  		=>  MemUdmxEO,     
    mcb3_dram_dm  		=>  MemLdmxEO,
    c3_clk0				=>  RefClkxC,
    c3_rst0				=>  open,
	--c3_calib_done      	=>  CalibDonexSN,
	c3_calib_done		=> CalibDonexS,
	-- termination
    mcb3_rzq   			=>  RzqxIO,
    mcb3_zio    		=>  ZioxIO,
     -- local interface to sampling core
	c3_p0_cmd_clk         =>  CoreClkxC,
    c3_p0_cmd_en          =>  WCmdValidxS,
    c3_p0_cmd_instr       =>  WCmdInstrxS,
    c3_p0_cmd_bl          =>  WBurstLenxD,
    c3_p0_cmd_byte_addr   =>  DramByteAdrxD,
    c3_p0_cmd_empty       =>  WCmdEmptyxS,
    c3_p0_cmd_full        =>  WCmdFullxS,
	
    c3_p0_wr_clk          =>  CoreClkxC,
    c3_p0_wr_en           =>  WritexSI,
    c3_p0_wr_mask         =>  "0000",	-- we always write full words so no bytes are masked out
    c3_p0_wr_data         =>  DataxDI,
    c3_p0_wr_full         =>  open,
    c3_p0_wr_empty        =>  WEmptyxSN,
    c3_p0_wr_count        =>  open,
    c3_p0_wr_underrun     =>  WUnderrunxS,
    c3_p0_wr_error        =>  WErrorxS,
    
	c3_p0_rd_clk          =>  '0',
    c3_p0_rd_en           =>  '0',
    c3_p0_rd_data         =>  open,
    c3_p0_rd_full         =>  open,
    c3_p0_rd_empty        =>  open,
    c3_p0_rd_count        =>  open,
    c3_p0_rd_overflow     =>  open,
    c3_p0_rd_error        =>  open,
	 
	c3_p1_cmd_clk         =>  WbClkxCI,
    c3_p1_cmd_en          =>  RCmdValidxS,
    c3_p1_cmd_instr       =>  "001",	-- we only send the 'read' command on this port
    c3_p1_cmd_bl          =>  RBurstLenxD,
    c3_p1_cmd_byte_addr   =>  ZERO_VECTOR(29-DRAM_CNTR_WIDTH downto 0) & SrcAdrxDP,
    c3_p1_cmd_empty       =>  RCmdEmptyxS,
    c3_p1_cmd_full        =>  RCmdFullxS,
	
    c3_p1_wr_clk          =>  '0',
    c3_p1_wr_en           =>  '0',
    c3_p1_wr_mask         =>  "0000",	-- we always read full words so no bytes are masked out
    c3_p1_wr_data         =>  x"00000000",
    c3_p1_wr_full         =>  open,
    c3_p1_wr_empty        =>  open,
    c3_p1_wr_count        =>  open,
    c3_p1_wr_underrun     =>  open,
    c3_p1_wr_error        =>  open,
    
	c3_p1_rd_clk          =>  WbClkxCI,
    c3_p1_rd_en           =>  RConsumexS,
    c3_p1_rd_data         =>  RDatOutxD,
    c3_p1_rd_full         =>  open,
    c3_p1_rd_empty        =>  REmptyxS,
    c3_p1_rd_count        =>  open,
    c3_p1_rd_overflow     =>  ROverflowxS,
    c3_p1_rd_error        =>  RErrorxS
);

end Behavioral;

