--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/06/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This Testbench tests the sampling core of BitHound. It initiates a full wishbone-dialog to reset and start
--* the core. The testbench is intended for manual inspection.
--*
--***************************************************************************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;
 
ENTITY SamplingCore_TB IS
END SamplingCore_TB;
 
ARCHITECTURE behavior OF SamplingCore_TB IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
  component la
    port (
      reset                : in  std_logic;
      clock                : in  std_logic;
      clockB               : in  std_logic;
      AVR_Clock            : in  std_logic;
      AVR_Reset            : in  std_logic;
      exClock              : in  std_logic;
      input                : in  std_logic_vector(31 downto 0);
      testOutput           : out std_logic;
      memoryOut            : out std_logic_vector(31 downto 0);
      memoryWrite          : out std_logic;
      Wb_cycxSI            : in  std_logic;
      Wb_stbxSI            : in  std_logic;
      Wb_wexSI             : in  std_logic;
      Wb_ackxSO            : out std_logic;
      Wb_errxSO            : out std_logic;
      Wb_adrxDI            : in  std_logic_vector(3 downto 0);
      Wb_datxDI            : in  std_logic_vector(7 downto 0);
      Wb_datxDO            : out std_logic_vector(7 downto 0);
      sample_finish_IRQxSO : out std_logic;
      sample_clrflagxSI    : in  std_logic);
  end component;

   --Inputs
   signal reset : std_logic := '1';
   signal clock : std_logic := '0';
   signal clockB : std_logic := '1';
   signal AVR_Clock : std_logic := '0';
   signal AVR_Reset : std_logic := '1';
   signal exClock : std_logic := '0';
   signal input : std_logic_vector(31 downto 0) := (others => '0');
   signal Wb_cycxSI : std_logic := '0';
   signal Wb_stbxSI : std_logic := '0';
   signal Wb_wexSI : std_logic := '0';
   signal Wb_adrxDI : std_logic_vector(3 downto 0) := (others => '0');
   signal Wb_datxDI : std_logic_vector(7 downto 0) := (others => '0');
   signal sample_clrflagxSI : std_logic := '0';

 	--Outputs
   signal testOutput : std_logic;
   signal memoryOut : std_logic_vector(31 downto 0);
   signal memoryWrite : std_logic;
   signal Wb_ackxSO : std_logic;
   signal Wb_errxSO : std_logic;
   signal sample_finish_IRQxSO : std_logic;

   -- Clock period definitions
   constant clock_period : time := 5 ns; --200MHz sampling clock
   constant AVR_Clock_period : time := 25 ns; --40MHz AVR clock
   constant exClock_period : time := 10 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: la PORT MAP (
          reset => reset,
          clock => clock,
          clockB => clockB,
          AVR_Clock => AVR_Clock,
          AVR_Reset => AVR_Reset,
          exClock => exClock,
          input => input,
          testOutput => testOutput,
          memoryOut => memoryOut,
          memoryWrite => memoryWrite,
          Wb_cycxSI => Wb_cycxSI,
          Wb_stbxSI => Wb_stbxSI,
          Wb_wexSI => Wb_wexSI,
          Wb_ackxSO => Wb_ackxSO,
          Wb_errxSO => Wb_errxSO,
          Wb_adrxDI => Wb_adrxDI,
          Wb_datxDI => Wb_datxDI,
          sample_finish_IRQxSO => sample_finish_IRQxSO,
          sample_clrflagxSI => sample_clrflagxSI
        );

   -- Clock process definitions
   clock_process :process
   begin
		clock <= '0';
		wait for clock_period/2;
		clock <= '1';
		wait for clock_period/2;
   end process;
   clockB <= not clock;
   
   AVR_Clock_process :process
   begin
		AVR_Clock <= '0';
		wait for AVR_Clock_period/2;
		AVR_Clock <= '1';
		wait for AVR_Clock_period/2;
   end process;
 
   exClock_process :process
   begin
		exClock <= '0';
		wait for exClock_period/2;
		exClock <= '1';
		wait for exClock_period/2;
   end process;
 
   --generate test-input (from lfsr)
	input(0) <= testOutput;

   -- Stimulus process
   stim_proc: process
   begin		
      
      wait for 300 ns;	
      reset <= '0';
      AVR_Reset <= '0';		
      wait for AVR_clock_period*5;
		wait until rising_edge(AVR_clock);
		
--reset core :
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"00"; --reset
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';

		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
--set flags:
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"82"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
--set sampling divider:
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"80"; --divide clock by 2
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"80"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
-- set trigger (to free running, or, rising edge on channel 0, or ...)

-- select trigger unit 0

             
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"C4"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
  -- set mask
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"C0"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
   -- set match value
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"01"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"C1"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
        -- set trigger flags
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"00";     -- select channel 0
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"10";     -- set serial and fire flag
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"C2"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);

--set trigger delay:
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"C3"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		
--set fwd:
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"08"; --take 8 samples! 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"83"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);

--set bwd:
		Wb_adrxDI <= b"0000"; 
		Wb_datxDI <= x"04"; --take 4 samples! 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0001"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0010"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0011"; 
		Wb_datxDI <= x"00"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);
		
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"84"; 
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';
		
		wait for AVR_clock_period*2;
		wait until rising_edge(AVR_clock);


--arm the logic analyzer:
		Wb_adrxDI <= b"0100"; --opcode
		Wb_datxDI <= x"01"; --run
		Wb_cycxSI <= '1';
		Wb_stbxSI <= '1';
		Wb_wexSI <= '1';
		wait until rising_edge(AVR_clock) and (Wb_ackxSO = '1' or Wb_errxSO = '1');
		Wb_cycxSI <= '0';
		Wb_stbxSI <= '0';

                --wait for AVR_clock_period*10;
                --wait until rising_edge(clock);
                --input     <= x"11111111";
		
--wait until the core asserts the IRQ-flag
		wait until sample_finish_IRQxSO = '1';
		wait for AVR_clock_period * 8;
		wait until rising_edge(AVR_clock);
		
--now, clear the flag:
      sample_clrflagxSI <= '1';
		wait for AVR_clock_period * 4;
		wait until rising_edge(AVR_clock);
		sample_clrflagxSI <= '0';
		

      wait;
   end process;

END;
