/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/17 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* BitHound firmware for ATmega103 core (this is NOT a whole uC!) on Spartan6 FPGA running at 40MHz.
--*
--***************************************************************************************************************/

#include <avr/interrupt.h>
#include "eth.h"
#include "uart.h"
#include "clock-arch.h"
#include "uip_arp.h"
#include "uip.h"
#include "timer.h"
#include "dhcpc.h"
#include "io.h"



/**************************************************************************************************
*   G L O B A L S
*/

// system tick counter
volatile uint32_t ticCnt=0;

// uIP data buffer
u8_t *uip_buf;

// ethernet address of the NIC
struct uip_eth_addr uipEthAddr;



/**************************************************************************************************
*   D E F I N E S
*/

#define BUF ((struct uip_eth_hdr *)&uip_buf[0])



/**************************************************************************************************
*   P R O T O T Y P E S
*/

void processRx(uint8_t bufferInd);



/**************************************************************************************************
*   I M P L E M E N T A T I O N 
*/

int main (void)
{
	uint8_t 	i;
	int8_t  	ind;
	uint32_t 	j;
	uint8_t 	led = 0x1;
	uint8_t		lastChannelLedUpdate = 0;
	uint8_t     lastEthLinkState = 255;
	//uint16_t 	len;
	
	struct timer periodic_timer, arp_timer;

	led = 0x01;	
	//LEDS = led;

	stdout = &mystdout;

	initUart();
	uartPrintStr_P(PSTR("hw init start\n"));
	//printf_P(PSTR("printf_P"));
	
	/* init spi */
	configSpi(0,0,16);	/* cpol = 0, cpha = 0, div clock by 16 */
	initLCD();
	//printUIntLCD(42);	
	
	ethInitPhy();
	ethInit();
	sei();

initEthLabel:
	led = 0x01;	
	LEDS = led;
	clrLCD();
	putsLCD("BitHound");

	/* wait until the phy established a ethernet connection */
	while(!ethGetLinkState());
		asm("nop");
	
	putsLCD("  eth up");
		
	// set up the ip stack
	timer_set(&periodic_timer, CLOCK_SECOND / 2);
	timer_set(&arp_timer, CLOCK_SECOND * 10);
	uip_init();
	
	// set ethernet address
 	struct uip_eth_addr e;
	e.addr[0] = OWN_MAC_ADDRESS_0;
	e.addr[1] = OWN_MAC_ADDRESS_1;
	e.addr[2] = OWN_MAC_ADDRESS_2;
	e.addr[3] = OWN_MAC_ADDRESS_3;
	e.addr[4] = OWN_MAC_ADDRESS_4;
	e.addr[5] = OWN_MAC_ADDRESS_5;
	uip_setethaddr 	( e ) ;

	// initialize the dhcp client and try to obtain an address
	dhcpc_init(&e, 6);
	
	uartPrintStr_P(PSTR("BitHound running\n"));
		
	while(1)
    {
		// check if we got new data
		ind = ethGetRxPacket();
		if (ind >= 0)
		{
			//led++;
			//LEDS = led;
			cli();
			uip_len = ethBufLen[ind];		// buffer contains data, process it
			sei();
			processRx(ind);
		}
			   
		i = ethGetLinkState();
		if (i != lastEthLinkState)
		{
			/*uartPrintStr("link: ");
			uartPrintUInt8(i);
			uartPutc('\n');*/
			lastEthLinkState = i;
			printf("link: %d\n",i);
			if (i==0)
			{
				/* we lost the ethernet connection, so we reset the MAC, stack and everthing above */
				appReset();	/* reset high level application to power up state */
				goto initEthLabel;
			}
		}
		
		// check the periodic timers
    	if(timer_expired(&periodic_timer)) 
		{
			// this happens twice every second
      		timer_reset(&periodic_timer);
			//uartPrintStr("timer\n");
			// the stack might generate a packet -> get a free transmit buffer
			do { ind = ethGetTxBuffer(); } while (ind<0);
			uip_buf = (uint8_t*)ethBuf[ind];
			// walk through all connections
      		for(i = 0; i < UIP_CONNS; i++) 
			{
				uip_periodic(i);
				/* If the above function invocation resulted in data that
	   			   should be sent out on the network, the global variable
	   			   uip_len is set to a value > 0. */
				if(uip_len > 0) 
				{
	  				uip_arp_out();
	  				ethSend(ind, uip_len);	// buffer number 2 is the transmit buffer
					// get a new buffer for the next connection
					do { ind = ethGetTxBuffer(); } while (ind<0);
					uip_buf = (uint8_t*)ethBuf[ind];
				}
			}
      	}

		/* Call the ARP timer function every 10 seconds. */
      	if(timer_expired(&arp_timer)) 
		{
			timer_reset(&arp_timer);
			uip_arp_timer();
      	}

#if UIP_UDP
		for(i = 0; i < UIP_UDP_CONNS; i++) 
	  	{
			// the stack might generate a packet -> set uip_buf to transmit buffer
			do { ind = ethGetTxBuffer(); } while (ind<0);
			uip_buf = (uint8_t*)ethBuf[ind];
			uip_udp_periodic(i);
			/* If the above function invocation resulted in data that
	   		   should be sent out on the network, the global variable
	   		   uip_len is set to a value > 0. */
			if(uip_len > 0)
			{
	  			//uartPrintStr("udp out\n");
				uip_arp_out();
	  			ethSend(ind, uip_len);	
				//ethWaitUntilSent(ind);
				do { ind = ethGetTxBuffer(); } while (ind<0);
				uip_buf = (uint8_t*)ethBuf[ind];
			}
      	}
#endif /* UIP_UDP */
      

		// update the channel leds everytime the clock ticks (i.e. with 100hz)
		j = clock_time();
		if ((j&0xff) != lastChannelLedUpdate)	// note we only compare the least significant byte to save calc power
		{
			//led++;
			//LEDS = led;
			lastChannelLedUpdate = (uint8_t)(j&0xff);
			j = LA_CHANNELS;	// get the current state of the channels
			//j = 0x55AAAAAA;
			// select the channel leds
			SPI_SEL_CH_LEDS();
			spiTransferByte((j>>24)&0xff);	// send the channel state Msb to Lsb
			spiTransferByte((j>>16)&0xff);
			spiTransferByte((j>>8)&0xff);
			spiTransferByte((j)&0xff);
			SPI_DESELECT();
		}
    }	// end of while(1)


}

/* passes data contained in buffer number bufferInd to the ip stack */
void processRx(uint8_t bufferInd)
{
	//uartPrintStr_P(PSTR("rx "));
	uip_buf = (uint8_t*) ethBuf[bufferInd];
	if (BUF->type == htons(UIP_ETHTYPE_IP))	// did we receive an ip packet?
	{
		/*uartPrintStr("ip ");
		printMac(uip_buf);
		uartPutc('\n');*/
		uip_arp_ipin();
		uip_input();
			/* If the above function invocation resulted in data that
		   should be sent out on the network, the global variable
		   uip_len is set to a value > 0. */
		if(uip_len > 0)
		{
			uip_arp_out();
		//	uartPrintStr("ip out\n");
			ethSend(bufferInd, uip_len);	// send buffer, it will be marked as free when the transmission is finished
		}
	} 
	else if(BUF->type == htons(UIP_ETHTYPE_ARP)) // did we receive an arp packet?
	{
		//uartPrintStr("arp \n");
		uip_arp_arpin();
		/* If the above function invocation resulted in data that
		   should be sent out on the network, the global variable
		   uip_len is set to a value > 0. */
		if(uip_len > 0)
		{
		//	uartPrintStr("arp out\n");
			ethSend(bufferInd, uip_len);	// send buffer, it will be marked as free when the transmission is finished
		}
	}
	
	// the packet was process, tell that the eth driver that we don't need it anymore
	ethRxConsume(bufferInd);
}

void uip_log(char *m)
{
	uartPrintStr(m);
	uartPutc('\n');
}

/* the dhcp client received a configuration -> inform the stack */
void dhcpc_configured(const struct dhcpc_state *s)
{
	uip_ipaddr_t ipaddr;
	// if s is null the dhcp client has given up -> default ip address
	if (s == NULL)
	{
		// set ip address
		uip_ipaddr(ipaddr, 10,0,0,2);
		uip_sethostaddr(ipaddr);
		// set default gateway
		uip_ipaddr(ipaddr, 10,0,0,1);
		uip_setdraddr(ipaddr);
		// set net mask
		uip_ipaddr(ipaddr, 255,0,0,0);
		uip_setnetmask(ipaddr);

		uartPrintStr_P(PSTR("dhcp has given up\n"));
		//LEDS = 0x02;

		/* show the address on the LC display */
		clrLCD();
		putsLCD("10.0.0.2");
	}
	else
	{
		uip_sethostaddr(s->ipaddr);
	  	uip_setnetmask(s->netmask);
  		uip_setdraddr(s->default_router);

		/* show the address on the LC display */
		clrLCD();
		printUIntLCD((s->ipaddr[0])&0xff);
		putLCD('.');
		printUIntLCD((s->ipaddr[0])>>8);
		putLCD('.');
		printUIntLCD((s->ipaddr[1])&0xff);
		putLCD('.');
		printUIntLCD((s->ipaddr[1])>>8);
		
		uartPrintStr_P(PSTR("got ip\n"));
		
		//LEDS = 0x01;
	}

	// init the high level application (this is where our protocol is decoded)
	appInit();
}

/* return the current system tick counter */
clock_time_t clock_time(void)
{
	uint32_t ret;
	cli();	// note: copying a 32 bit value is not an atomic operation on AVR -> block interrupts
	ret = ticCnt;
	sei();
	return ret;
}

/* system timer, generated periodically */
ISR (INT0_vect)
{
	ticCnt++;
}

